//
//  JabberIdentity.m
//  Jabber
//
//  Created by David Chisnall on 20/08/2004.
//  Copyright 2004 __MyCompanyName__. All rights reserved.
//

#import "JabberIdentity.h"
#import "JabberRootIdentity.h"
#import "TRXMLString.h"
#include "../Macros.h"

@implementation JabberIdentity


- (id) initWithJID:(JID*)_jid withName:(NSString*)_name inGroup:(NSString*)_group forPerson:(id)_person
{
	[self init];
	jid = [_jid retain];
	name = [_name retain];
	group = [_group retain];
	person = [_person retain];
	return self;
}

- (id) init
{
	SUPERINIT;
	presence = [[Presence alloc] init];
	return self;
}

/*
 <query xmlns='jabber:iq:roster'>
 <item
 jid='user@example.com'
 subscription='to'
 name='SomeUser'>
 <group>SomeGroup</group>
 </item>
 </query>
 */

- (void)startElement:(NSString *)aName
		  attributes:(NSDictionary*)attributes
{
	if([aName isEqualToString:@"item"])
	{
		depth++;
		jid = [[JID alloc] initWithString:[attributes objectForKey:@"jid"]];
		subscription = [[attributes objectForKey:@"subscription"] retain];
		name = [[attributes objectForKey:@"name"] retain];
	}
	else if([aName isEqualToString:@"group"])
	{
		[[[TRXMLString alloc] initWithXMLParser:parser
										parent:self
										   key:@"group"] startElement:aName
														   attributes:attributes];
	}
	else
	{
		[[[TRXMLNullHandler alloc] initWithXMLParser:parser
											  parent:self
												 key:nil] startElement:aName
															attributes:attributes];
	}
}

- (void) addgroup:(NSString*)aGroup
{
	[group release];
	group = [aGroup retain];
}

- (void) setPresence:(Presence*)_presence
{
	[presence release];
	presence = [_presence retain];
	priority = basePriority + 70 - [presence show] + [presence priority];	
}

- (NSString*) name
{
	if (name == nil)
	{
		return [jid node];
	}
	return name;
}

- (NSString*) group
{
	return group;
}

- (JID*) jid
{
	return jid;
}

- (Presence*) presence
{
	return presence;
}

- (int) priority
{
	return priority;
}

- (id) person
{
	return person;
}

- (void) person:(id)_person
{
	[person release];
	person = [_person retain];
}

- (NSComparisonResult) compareByPriority:(JabberIdentity*)_other
{
	if(priority > [_other priority])
	{
		return NSOrderedAscending;
	}
	if(priority < [_other priority])
	{
		return NSOrderedDescending;
	}
	return NSOrderedSame;
}

- (NSComparisonResult) compareByJID:(JabberIdentity*)_other
{
	return NSOrderedAscending;
}

- (void) dealloc
{
	[person release];
	[jid release];
	[subscription release];
	[group release];
	[name release];
	[presence release];
	[super dealloc];
}
@end
